let _Vue = null
export default class VueRouter {
  static install(Vue) {
    // 判断当前插件是否被安装
    if( VueRouter.install.installed){
      return
    }
    VueRouter.install.installed = true
    // 把Vue构造函数记录到全局变量
    _Vue = Vue
    // 把创建Vue实例时候传入的router对象注入到vue实例中
    // 
    // 混入
    _Vue.mixin({
      beforeCreate() {
        if(this.$options.router){
          _Vue.prototype.$router = this.$options.router
          this.$options.router.init()
        }
      }
    })
  }

  constructor(options){
    this.options = options
    this.routeMap = {}
    this.data = _Vue.observable({
      current:  window.location.pathname
    }) 
  }

  init(){
    this.createRouteMap()
    this.initComponents(_Vue)
    this.initEvent()
  }

  createRouteMap(){
    // 遍历素有的路由规则，把路由规则解析成键值对形式，储存到routeMap中
    this.options.routes.forEach(route=>{
      this.routeMap[route.path] = route.component
    })
  }

  initComponents(Vue){
    Vue.component('router-link', {
      props:{
        to: String
      },
      // template:'<a :href="to"><slot></slot></a>'
      render(h){
        return h('a', {
          attrs:{
            href: this.to
          },
          on:{
            click: this.clickHandler
          }
        }, [this.$slots.default])
      },
      methods:{
        clickHandler(e){
          history.pushState({},'', this.to)
          this.$router.data.current = this.to
          e.preventDefault();
        }
      }
    })

    const self = this
    Vue.component('router-view', {
      render(h){
        const component = self.routeMap[self.data.current]
        return h(component)
      }
    })
  }

  initEvent(){
    window.addEventListener('popstate',()=>{
      this.data.current = window.location.pathname
    })
  }
}